package edu.uclm.esi.mongo.isotel.dominio;

import java.text.ParseException;
import java.util.GregorianCalendar;
import java.util.Vector;

import edu.uclm.esi.mongo.etiquetasMongo.Bsonable;
import edu.uclm.esi.mongo.isotel.persistencia.DAOCliente;
import edu.uclm.esi.mongo.isotel.persistencia.DAOLlamada;
import edu.uclm.esi.mongo.isotel.persistencia.DAOTarifa;

public class Cliente {
	private int idCliente;
	@Bsonable
	private String nombre;
	@Bsonable
	private String apellido1;
	@Bsonable
	private String apellido2;
	@Bsonable
	private String dni;
	@Bsonable(index = true)
	private String telefono;
	@Bsonable
	private GregorianCalendar fechaDeNacimiento;
	private Tarifa tarifa;
	private Vector<Llamada> llamadas;
	
	public Cliente() {
	}

	public Cliente(int idCliente) {
		DAOCliente.load(idCliente, this);
	}

	public Cliente(int idCliente, String nombre, String apellido1, String apellido2, String dni, String telefono, GregorianCalendar fechaDeNacimiento) {
		this.idCliente=idCliente;
		this.nombre=nombre;
		this.apellido1=apellido1;
		this.apellido2=apellido2;
		this.dni=dni;
		this.telefono=telefono;
		this.fechaDeNacimiento=fechaDeNacimiento;
		this.llamadas=new Vector<>();
	}
	
	public void update() {
		DAOCliente.update(this);
	}

	public int getId() {
		return idCliente;
	}

	public String getNombre() {
		return nombre;
	}

	public String getApellido1() {
		return apellido1;
	}
	
	public String getApellido2() {
		return apellido2;
	}

	public String getDNI() {
		return dni;
	}

	public String getTelefono() {
		return telefono;
	}
	
	public GregorianCalendar getFechaDeNacimiento() {
		return fechaDeNacimiento;
	}

	public Tarifa getTarifa() {
		if (this.tarifa==null)
			this.tarifa=DAOTarifa.loadTarifa(this.idCliente);
		return this.tarifa;
	}

	public void addLlamada(Llamada llamada) {
		this.llamadas.add(llamada);
	}

	public void setTarifa(Tarifa t) {
		this.tarifa=t;
	}
	
	public void setTarifa(String idTarifa) {
		Class<Tarifa> ClaseTarifa;
		try {
			ClaseTarifa = (Class<Tarifa>) Class.forName("edu.uclm.esi.mongo.isotel.dominio." + idTarifa);
			this.tarifa = ClaseTarifa.newInstance();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public Vector<Llamada> getLlamadas(String desde, String hasta) throws ParseException {
		return DAOLlamada.getLlamadas(idCliente, desde, hasta);
	}

	public Vector<Llamada> getLlamadas() {
		if (llamadas==null || llamadas.size()==0)
			 this.llamadas=DAOLlamada.getLlamadas(idCliente);
		return llamadas;
	}

	public void setIdCliente(int idCliente) {
		this.idCliente = idCliente;
	}

	public void setNombre(String nombre) {
		this.nombre = nombre;
	}

	public void setApellido1(String apellido1) {
		this.apellido1 = apellido1;
	}

	public void setApellido2(String apellido2) {
		this.apellido2 = apellido2;
	}

	public void setDni(String dni) {
		this.dni = dni;
	}

	public void setTelefono(String telefono) {
		this.telefono = telefono;
	}

	public void setFechaDeNacimiento(GregorianCalendar fechaDeNacimiento) {
		this.fechaDeNacimiento = fechaDeNacimiento;
	}

	public void setLlamadas(Vector<Llamada> llamadas) {
		this.llamadas = llamadas;
	}
}
