package edu.uclm.esi.mongo.isotel.persistencia;

import java.util.GregorianCalendar;
import java.util.Vector;

import org.bson.BsonDateTime;
import org.bson.BsonDocument;
import org.bson.BsonInt32;
import org.bson.BsonString;

import com.mongodb.BasicDBObject;
import com.mongodb.DBCollection;
import com.mongodb.DBObject;
import com.mongodb.MapReduceCommand;
import com.mongodb.MapReduceOutput;
import com.mongodb.client.MongoCollection;
import com.mongodb.client.MongoCursor;
import com.mongodb.client.result.DeleteResult;

import edu.uclm.esi.mongo.isotel.dominio.Cliente;

public class DAOCliente {
	
	public static Cliente findCliente(String sIdCliente, String dni, String nombre, String apellido1, String apellido2) throws Exception {
		BsonDocument criterioBusqueda=new BsonDocument();
		if (sIdCliente!=null && sIdCliente.length()>0) {
			criterioBusqueda.append("_id", new BsonInt32(Integer.parseInt(sIdCliente)));
		}
		if (dni!=null && dni.length()>0) {
			criterioBusqueda.append("dni", new BsonString(dni));
		}
		if (nombre!=null && nombre.length()>0) {
			criterioBusqueda.append("nombre", new BsonString(nombre.toUpperCase()));
		}
		if (apellido1!=null && apellido1.length()>0) {
			criterioBusqueda.append("apellido1", new BsonString(apellido1.toUpperCase()));
		}
		if (apellido2!=null && apellido2.length()>0) {
			criterioBusqueda.append("apellido2", new BsonString(apellido2.toUpperCase()));
		}
		if (criterioBusqueda.keySet().size()==0)
			throw new Exception("Introduzca al menos un criterio de búsqueda");
		MongoCollection<BsonDocument> colClientes=MongoBroker.get().getDB().getCollection("clientes", BsonDocument.class);
		if (colClientes.count(criterioBusqueda)==0)
			throw new Exception("No hay clientes que cumplan el criterio de búsqueda");
		if (colClientes.count(criterioBusqueda)>1)
			throw new Exception("Hay más de un cliente que cumple el criterio de búsqueda. Por favor, refínelo");
		BsonDocument bso=colClientes.find(criterioBusqueda).first();
		Cliente result=new Cliente(bso.get("_id").asInt32().getValue());
		return result;
	}
	
public static Vector<Cliente> getClientes(int inicio, int n) {
	MongoCollection<BsonDocument> colClientes=
			MongoBroker.get().getDB().getCollection("clientes", BsonDocument.class);
	BsonDocument criterioOrdenacion=new BsonDocument("apellido1", new BsonInt32(1));
	criterioOrdenacion.append("apellido2", new BsonInt32(1));
	criterioOrdenacion.append("nombre", new BsonInt32(1));
	
	MongoCursor<BsonDocument> clientes = colClientes.find().sort(criterioOrdenacion).
			skip(inicio).limit(10).iterator();
	Vector<Cliente> result=new Vector<>();
	int id;
	String nombre, apellido1, apellido2, dni, telefono;
	GregorianCalendar fechaDeNacimiento;
	Cliente cliente;
	BsonDocument bso;
	while (clientes.hasNext()) {
		bso=clientes.next();
		id=bso.get("_id").asInt32().getValue();
		nombre=bso.get("nombre").asString().getValue();
		apellido1=bso.get("apellido1").asString().getValue();
		apellido2=bso.get("apellido2").asString().getValue();
		dni=bso.get("dni").asString().getValue();
		telefono=bso.get("telefono").asString().getValue();
		fechaDeNacimiento=new GregorianCalendar(); 
		fechaDeNacimiento.setTimeInMillis(bso.get("fechaDeNacimiento").asDateTime().getValue());
		cliente=new Cliente(id, nombre, apellido1, apellido2, dni, telefono, fechaDeNacimiento);
		result.add(cliente);
	}
	return result;
}

	public static void load(int idCliente, Cliente cliente) {
		MongoCollection<BsonDocument> colClientes=MongoBroker.get().getDB().getCollection("clientes", BsonDocument.class);
		BsonDocument criterio=new BsonDocument("_id", new BsonInt32(idCliente));
		
		BsonDocument bso = colClientes.find(criterio).first();
		int id;
		String nombre, apellido1, apellido2, dni, telefono;
		GregorianCalendar fechaDeNacimiento;
		id=bso.get("_id").asInt32().getValue();
		nombre=bso.get("nombre").asString().getValue();
		apellido1=bso.get("apellido1").asString().getValue();
		apellido2=bso.get("apellido2").asString().getValue();
		dni=bso.get("dni").asString().getValue();
		telefono=bso.get("telefono").asString().getValue();
		fechaDeNacimiento=new GregorianCalendar(); 
		fechaDeNacimiento.setTimeInMillis(bso.get("fechaDeNacimiento").asDateTime().getValue());
		cliente.setIdCliente(id);
		cliente.setNombre(nombre);
		cliente.setApellido1(apellido1); cliente.setApellido2(apellido2);
		cliente.setDni(dni); cliente.setTelefono(telefono);
		cliente.setFechaDeNacimiento(fechaDeNacimiento);
	}

	public static void update(Cliente cliente) {
		MongoCollection<BsonDocument> colClientes=MongoBroker.get().getDB().getCollection("clientes", BsonDocument.class);
		BsonDocument criterio=new BsonDocument("_id", new BsonInt32(cliente.getId()));
		BsonDocument criterioActualizacion=new BsonDocument("dni", new BsonString(cliente.getDNI()));
		criterioActualizacion.append("nombre", new BsonString(cliente.getNombre()));
		criterioActualizacion.append("apellido1", new BsonString(cliente.getApellido1()));
		criterioActualizacion.append("apellido2", new BsonString(cliente.getApellido2()));
		criterioActualizacion.append("telefono", new BsonString(cliente.getTelefono()));
		criterioActualizacion.append("fechaDeNacimiento", new BsonDateTime(cliente.getFechaDeNacimiento().getTimeInMillis()));
		criterioActualizacion.append("idTarifa", new BsonString(cliente.getTarifa().getClass().getSimpleName()));
		colClientes.replaceOne(criterio, criterioActualizacion);
	}
	
	public static long[] delete(int idCliente) {
		MongoCollection<BsonDocument> colClientes=MongoBroker.get().getDB().getCollection("clientes", BsonDocument.class);
		BsonDocument criterio=new BsonDocument("_id", new BsonInt32(idCliente));
		DeleteResult resultCliente = colClientes.deleteOne(criterio);
		
		MongoCollection<BsonDocument> colLlamadas=MongoBroker.get().getDB().getCollection("llamadas", BsonDocument.class);
		criterio=new BsonDocument("idCliente", new BsonInt32(idCliente));
		DeleteResult resultLlamadas=colLlamadas.deleteMany(criterio);
		return new long[] {resultCliente.getDeletedCount(), resultLlamadas.getDeletedCount()};
	}
}
