package edu.uclm.esi.mongo.isotel.persistencia;

import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import org.bson.BsonDocument;
import org.bson.BsonDouble;
import org.bson.BsonInt32;
import org.bson.BsonJavaScript;
import org.bson.BsonString;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;

import com.mongodb.DBCollection;
import com.mongodb.MapReduceCommand;
import com.mongodb.MapReduceCommand.OutputType;
import com.mongodb.MapReduceOutput;
import com.mongodb.client.MapReduceIterable;
import com.mongodb.client.MongoCollection;
import com.mongodb.client.MongoCursor;
import com.mongodb.client.model.MapReduceAction;

import edu.uclm.esi.mongo.isotel.dominio.Factura;
import edu.uclm.esi.mongo.isotel.dominio.Tarifa;

public class DAOFactura {
	public static Vector<Factura> facturarMR(int mes, int year) throws Exception {
		MongoCollection<BsonDocument> colScripts=MongoBroker.get().getDB().getCollection("scripts", BsonDocument.class);
		BsonDocument criterio=new BsonDocument("_id", new BsonString("llamadasDeUnMes"));
		String map=colScripts.find(criterio).first().get("codigo").asJavaScript().getCode();
		
		criterio=new BsonDocument("_id", new BsonString("facturacion"));
		String reduce=colScripts.find(criterio).first().get("codigo").asJavaScript().getCode();
		
		MongoCollection<BsonDocument> colLlamadas=MongoBroker.get().getDB().getCollection("llamadas", BsonDocument.class);
		
		MapReduceIterable<BsonDocument> reduccion = colLlamadas.mapReduce(map, reduce);
		reduccion=reduccion.collectionName("facturacion"+year+"_"+mes);
		BsonDocument scope=new BsonDocument();
		scope.append("year", new BsonInt32(year));
		scope.append("mes", new BsonInt32(mes-1));
		reduccion=reduccion.scope(scope);
		reduccion=reduccion.action(MapReduceAction.REPLACE);
		reduccion.first();
		
		MongoCollection<BsonDocument> colFacturacion=MongoBroker.get().getDB().getCollection("facturacion"+year+"_"+mes, BsonDocument.class);
		MongoCursor<BsonDocument> facturas = colFacturacion.find().iterator();
		BsonDocument bso;
		int idCliente, numeroDeLlamadas, duracionTotal;
		Tarifa tarifa;
		Vector<Factura> result=new Vector<>();
		while (facturas.hasNext()) {
			bso=facturas.next();
			idCliente=(int) bso.get("_id").asDouble().getValue();
			numeroDeLlamadas=(int) bso.get("value").asDocument().get("numeroDeLlamadas").asDouble().getValue();
			duracionTotal=(int) bso.get("value").asDocument().get("duracionTotal").asDouble().getValue();
			tarifa=DAOTarifa.loadTarifa(idCliente);
			Factura factura=new Factura(idCliente, numeroDeLlamadas, duracionTotal, tarifa, mes, year);
			factura.insert();
			result.add(factura);
		}
		
		return result;
	}
	
	@SuppressWarnings("unchecked")
	public static JSONArray getFacturas(int mes, int year, int inicio) {
		MongoCollection<BsonDocument> colFacturas=MongoBroker.get().getDB().getCollection("facturas", BsonDocument.class);
		BsonDocument criterioSeleccion=new BsonDocument("mes", new BsonInt32(mes));
		criterioSeleccion.append("year", new BsonInt32(year));
		
		BsonDocument criterioOrdenacion=new BsonDocument("idCliente", new BsonInt32(1));
		
		MongoCursor<BsonDocument> facturas = colFacturas.find(criterioSeleccion).sort(criterioOrdenacion).
				skip(inicio).limit(10).iterator();
		JSONObject factura;
		BsonDocument bso;
		JSONArray result=new JSONArray();
		while (facturas.hasNext()) {
			bso=facturas.next();
			factura=new JSONObject();			
			factura.put("_id", bso.get("_id").asObjectId().getValue());
			factura.put("idCliente", bso.get("idCliente").asInt32().getValue());
			factura.put("dni", bso.get("dni").asString().getValue());
			factura.put("nombre", bso.get("nombre").asString().getValue());
			factura.put("apellido1", bso.get("apellido1").asString().getValue());
			factura.put("apellido2", bso.get("apellido2").asString().getValue());
			factura.put("cuotaFija", bso.get("cuotaFija").asDouble().getValue());
			factura.put("establecimientosDeLlamada", bso.get("establecimientosDeLlamada").asDouble().getValue());
			factura.put("importePorConsumo", bso.get("importePorConsumo").asDouble().getValue());
			factura.put("tipoDeTarifa", bso.get("tipoDeTarifa").asString().getValue());
			result.add(factura);
		}
		return result;	
	}
	
	public static long getNumeroDeLlamadas(int mes, int year) throws Exception {
		MongoCollection<BsonDocument> colLlamadas=MongoBroker.get().getDB().getCollection("llamadas", BsonDocument.class);
		String code="function() { " +
				"return this.fecha.getMonth()==" + (mes-1) + " && this.fecha.getFullYear()==" + year +";" +
				"}";
		BsonDocument criterio=new BsonDocument("$where", new BsonJavaScript(code));
		return colLlamadas.count(criterio);
	}

	public static void insert(Factura factura) {
		MongoCollection<BsonDocument> colFacturas=MongoBroker.get().getDB().getCollection("facturas", BsonDocument.class);
		BsonDocument bso=new BsonDocument();
		bso.append("idCliente", new BsonInt32(factura.getCliente().getId()));
		bso.append("dni", new BsonString(factura.getCliente().getDNI()));
		bso.append("nombre", new BsonString(factura.getCliente().getNombre()));
		bso.append("apellido1", new BsonString(factura.getCliente().getApellido1()));
		bso.append("apellido2", new BsonString(factura.getCliente().getApellido2()));
		bso.append("mes", new BsonInt32(factura.getMes()));
		bso.append("year", new BsonInt32(factura.getYear()));
		bso.append("cuotaFija", new BsonDouble(factura.getCuotaFija()));
		bso.append("establecimientosDeLlamada", new BsonDouble(factura.getCostesEstablecimiento()));
		bso.append("importePorConsumo", new BsonDouble(factura.getCostesPorConsumo()));
		bso.append("tipoDeTarifa", new BsonString(factura.getTarifa().getClass().getSimpleName()));
		colFacturas.insertOne(bso);
	}
}
