package edu.uclm.esi.mongo.isotel.persistencia;

import java.text.ParseException;
import java.util.GregorianCalendar;
import java.util.Vector;

import org.bson.BsonDateTime;
import org.bson.BsonDocument;
import org.bson.BsonInt32;
import org.bson.types.ObjectId;

import com.mongodb.client.MongoCollection;
import com.mongodb.client.MongoCursor;

import edu.uclm.esi.mongo.isotel.auxiliares.Fechas;
import edu.uclm.esi.mongo.isotel.dominio.Llamada;

public class DAOLlamada {
	
	public static Vector<Llamada> getLlamadas(int idCliente) {
		MongoCollection<BsonDocument> colLlamadas=MongoBroker.get().getDB().getCollection("llamadas", BsonDocument.class);
		BsonDocument criterioConsulta=new BsonDocument("idCliente", new BsonInt32(idCliente));
		BsonDocument criterioOrdenacion=new BsonDocument("fecha", new BsonInt32(-1));
		MongoCursor<BsonDocument> llamadas=colLlamadas.find(criterioConsulta).sort(criterioOrdenacion).iterator();
		BsonDocument bso;
		ObjectId idLlamada;
		int duracion;
		String origen, destino;
		GregorianCalendar fecha;
		Llamada llamada;
		Vector<Llamada> result=new Vector<>();
		while (llamadas.hasNext()) {
			bso=llamadas.next();
			idLlamada=bso.get("_id").asObjectId().getValue();
			origen=bso.get("origen").asString().getValue();
			destino=bso.get("destino").asString().getValue();
			duracion=bso.get("duracion").asInt32().getValue();
			fecha=new GregorianCalendar();
			fecha.setTimeInMillis(bso.get("fecha").asDateTime().getValue());
			llamada=new Llamada(idLlamada, idCliente, origen, destino, duracion, fecha);
			result.add(llamada);
		}
		return result;
	}

	public static Vector<Llamada> getLlamadas(int idCliente, String sDesde, String sHasta) throws ParseException {
		GregorianCalendar desde=Fechas.getFecha(sDesde);
		GregorianCalendar hasta=Fechas.getFecha(sHasta);

		MongoCollection<BsonDocument> colLlamadas=MongoBroker.get().getDB().getCollection("llamadas", BsonDocument.class);
		BsonDocument criterioConsulta=new BsonDocument("idCliente", new BsonInt32(idCliente));
		BsonDocument intervaloFechas=new BsonDocument();
		intervaloFechas.append("$gte", new BsonDateTime(desde.getTimeInMillis()));
		intervaloFechas.append("$lte", new BsonDateTime(hasta.getTimeInMillis()+24*3600*1000));
		
		criterioConsulta.append("fecha", intervaloFechas);
		BsonDocument criterioOrdenacion=new BsonDocument("fecha", new BsonInt32(-1));
		MongoCursor<BsonDocument> llamadas=colLlamadas.find(criterioConsulta).sort(criterioOrdenacion).iterator();
		BsonDocument bso;
		ObjectId idLlamada;
		int duracion;
		String origen, destino;
		GregorianCalendar fecha;
		Llamada llamada;
		Vector<Llamada> result=new Vector<>();
		while (llamadas.hasNext()) {
			bso=llamadas.next();
			idLlamada=bso.get("_id").asObjectId().getValue();
			origen=bso.get("origen").asString().getValue();
			destino=bso.get("destino").asString().getValue();
			duracion=bso.get("duracion").asInt32().getValue();
			fecha=new GregorianCalendar();
			fecha.setTimeInMillis(bso.get("fecha").asDateTime().getValue());
			llamada=new Llamada(idLlamada, idCliente, origen, destino, duracion, fecha);
			result.add(llamada);
		}
		return result;
	}
}
